module VNA (
	input clock,
	input eth_nRESET,
	input signed [15:0] adc_data,
	input signed [15:0] tx_cordic_out_i,
	input signed [15:0] tx_cordic_out_q,
	input [15:0] vna_count,
	input [31:0] rx_tune_phase,
	input [31:0] tx_tune_phase,
	output reg [31:0] vna_tune_phase,
	output reg vna_avail,
	output reg signed [23:0] vna_real,
	output reg signed [23:0] vna_imag
	);

	parameter CountWait = 14'd8000;
	parameter CountData = 14'd4096;
	reg [13:0] counter;
	reg [15:0] npixels;
	reg signed [33:0] sum_real, sum_imag, vna_data_real, vna_data_imag;

	// State machine for VNA
	reg [3:0] vna_state;
	parameter sVna0		= 0;
	parameter sVna1		= 1;
	parameter sVna2		= 2;
	parameter sVna3		= 3;
	parameter sVna4		= 4;
	parameter sVna5		= 5;
	parameter sVna1_2		= 6;

	always @(posedge clock)		// VNA state machine
	begin
		if (eth_nRESET == 0 || vna_count == 0)	// we are in reset, or non-VNA
		begin
			vna_state <= sVna0;
			counter <= CountWait;
			vna_tune_phase <= 1'd0;
			npixels <= vna_count;
		end
		else case (vna_state)
		sVna0:		// Throw away data after frequency change
		begin
			if (counter)
			begin
				counter <= counter - 1'd1;
			end
			else
			begin
				vna_state <= sVna1;
				counter <= CountData;
			end
		end
		sVna1:		// Sum data
		begin
			if (counter)
			begin
				counter <= counter - 1'd1;
				vna_data_real <= tx_cordic_out_q * (adc_data >>> 6);
			   vna_data_imag <= tx_cordic_out_i * (adc_data >>> 6);
				vna_state <= sVna1_2;
			end
			else
			begin
				vna_state <= sVna2;
				counter <= CountWait;
			end
		end
		sVna1_2:
		begin
			sum_real <= sum_real + vna_data_real;
			sum_imag <= sum_imag + vna_data_imag;
			vna_state <= sVna1;
		end
		sVna2:		// Send an output datum
		begin
			vna_avail <= 1;
			vna_real <= sum_real[33:10];
			vna_imag <= sum_imag[33:10];
			vna_state <= sVna3;
		end
		sVna3:		// Increment the frequency, and repeat
		begin
			vna_avail <= 0;
			sum_real <= 1'd0;
			sum_imag <= 1'd0;
			vna_data_real <= 1'd0;
			vna_data_imag <= 1'd0;
			if (npixels != 1'd1)
			begin
				npixels <= npixels - 1'd1;
				vna_tune_phase <= vna_tune_phase + tx_tune_phase;
				vna_state <= sVna0;
			end
			else
			begin
				npixels <= vna_count;
				vna_tune_phase <= rx_tune_phase;
				vna_state <= sVna4;
			end
		end
		sVna4:		// Frequency reset to start, send a zero sample to mark the start
		begin
			if (counter)
			begin
				counter <= counter - 1'd1;
			end
			else
			begin
				vna_avail <= 1'd1;
				vna_real <= 1'd0;
				vna_imag <= 1'd0;
				vna_state <= sVna5;
			end
		end
		sVna5:
		begin
			vna_avail <= 1'd0;
			counter <= CountWait;
			vna_state <= sVna0;
		end
		endcase
	end
endmodule