//////////////////////////////////////////////////////////////////////////////////
// Module for TLV320AIC23 codec
// 
// Mode Master I2S 
// 
// David Fainitski for Odyssey TRX
// 06.02.15   
//
//////////////////////////////////////////////////////////////////////////////////

module sound_codec(
   input clock_122M,
	input reset,
	input signed [31:0] cdc_sample,		// next sample to codec
	output reg get_next_cdc,
	input pro_clock, // 96kHz
	input st_on,
	input [7:0] st_volume,
	output reg PWM_1,
	output reg PWM_2,
	input [111:0] key_ctrl,
	input key_ctrl_valid,
	input PTT_SSB,
	input PTT_CW,
	input tune,
	input tx_en
	);
	
	//
	reg [3:0] state_tone = 0;
	reg [9:0] pro_count = 0; 
	reg [16:0] sd_timer = 0;
	reg tone_en = 0;
	always @(negedge pro_clock)
	begin
	   case (state_tone)
		0: begin 
		      tone_en <= 0;
		      if (pro_count != 511) pro_count <= pro_count + 1'd1;
				if (st_on) state_tone <= 1;
				else if (PTT_SSB) state_tone <= 7;
				else if (PTT_CW) state_tone <= 4;
		   end		
		1: begin
				if (pro_count != 0) pro_count <= pro_count - 1'd1;
				else state_tone <= 2; 
		   end
		2: begin
		      tone_en <= 1;
		      if (pro_count != 511) pro_count <= pro_count + 1'd1;
				else state_tone <= 3;
	      end	
		3: if(st_on == 0) state_tone <= 4;
	   4: begin
				if (pro_count != 0) pro_count <= pro_count - 1'd1;
				else state_tone <= 5;
	      end	
      5: begin   //hang delay
				sd_timer <= hang * 8'd96;
				state_tone <= 6;
         end		
		6: begin
		      if (st_on) state_tone <= 2;
				else if (sd_timer == 0) state_tone <= 0;  
				else if (PTT_CW) state_tone <= 5;
				else sd_timer <= sd_timer - 1'd1; 
	      end
		7: begin
		      if (pro_count != 0) pro_count <= pro_count - 1'd1;
				else state_tone <= 8; 
			end	
		8: begin
		     if (PTT_SSB == 0) begin state_tone <= 8'd9; sd_timer <= hang * 8'd96;  end		
	      end
		9: begin
		      if (sd_timer != 0) sd_timer <= sd_timer - 1'd1;
				else state_tone <= 10;
	      end 	
		10: begin
		       if (pro_count != 511) pro_count <= pro_count + 1'd1;
				 else state_tone <= 0;
			 end	 	
		default: state_tone <= 0;
		endcase
	end
	
	
	 // Sound profiling
	 wire [15:0] s_profile;
	 profile sine_profile (.address(pro_count), .clock(pro_clock), .q(s_profile));
	 
	 // Side Tone Volume
	 wire [15:0] volume = tone_en  ? {1'b0,st_volume, 7'b0} : 16'd32767;
	 
	 mult_16x16 tone_volume_l (pro_sound_left,  volume, sound_cdc_left);
	 mult_16x16 tone_volume_r (pro_sound_right, volume, sound_cdc_right);
	 
	 wire signed [15:0] pro_sound_left, pro_sound_right;
	 wire signed [15:0] sound_cdc_left, sound_cdc_right;
	 wire signed [15:0] source_left =  (~tune && tx_en) ? (tone_en ? sine_tone : cdc_sample[31:16]) : 16'd0;
	 wire signed [15:0] source_right = (~tune && tx_en) ? (tone_en ? sine_tone : cdc_sample[15: 0]) : 16'd0;
	 
	 mult_16x16 tone_pro_l (source_left,  s_profile, pro_sound_left);
	 mult_16x16 tone_pro_r (source_right, s_profile, pro_sound_right);
	  
	 // Tone Freq Calculating 				
   reg [11:0] tone_freq = 700;
	reg [11:0] hang = 230;
   always @(posedge key_ctrl_valid) 
	begin 
	   if (reset) 
		   begin 
			   tone_freq <= key_ctrl[8*6 +: 8] * 8'd10;
				hang  <= key_ctrl[8*8 +: 8]*8'd10 + key_ctrl[8*7 +: 8]; 
	      end			
   end		
					  
   reg sidetone_clock;
   reg [9:0] side_count; 
   wire [15:0] sidetone_divide;
   wire signed [15:0] prof_sidetone;

   // calculate clock for sidetone generator 
   // sidetone_divide = (122880000/(1000 * tone_freq))/2 - 1 = (61440/tone_freq) - 1

   divide_f	divide_freq (.denom (tone_freq), .numer(61440), .quotient(sidetone_divide));

   // generate clock 
   always @ (posedge clock_122M)
   begin
	   if (side_count == (sidetone_divide - 16'd1)) begin
		   side_count <= 0;
		   sidetone_clock <= ~sidetone_clock;
	   end 
	   else side_count <= side_count + 10'd1;
   end		
		
	// Tone generating
	reg [9:0] tone_count = 0;
	wire signed [15:0] sine_tone;
	sine_table si_tone (.address(tone_count), .clock(sidetone_clock), .q(sine_tone));
	always @(negedge sidetone_clock) 
	begin
		if (tone_count != 999) tone_count <= tone_count + 1'd1;
		else tone_count <= 0;
	end 
	//
// PWM output
	reg [1:0] d;
	reg [11:0] s_buff_1, s_buff_2;
	always @(posedge pro_clock) // 96kHz
	begin 
	   d <= d + 1'd1; 
	   if(d[1]) // 24kHz
	   begin
		   s_buff_1 <= {~sound_cdc_left[15], sound_cdc_left[14:4]};
		   s_buff_2 <= {~sound_cdc_right[15], sound_cdc_right[14:4]};
	   end
		if(d[0]) get_next_cdc <= ~get_next_cdc; // 48kHz
	end
	//
	reg [11:0] pwm_cnt;
	reg [11:0] s_buff_3, s_buff_4;
	always @(posedge clock_122M)
	begin
	   if(!reset)
		begin
		   PWM_1 <= 0;
			PWM_2 <= 0;
			pwm_cnt <= 0;
		end
		else
		begin
		   pwm_cnt <= pwm_cnt + 1'd1;
			if(pwm_cnt==0)
			begin
			   s_buff_3 <= s_buff_1;
		      s_buff_4 <= s_buff_2;
			end
	      PWM_1 <= s_buff_3 > pwm_cnt;
		   PWM_2 <= s_buff_4 > pwm_cnt;
		end
	end
	//
endmodule

