// (C) Phil Harman VK6APH 2006, 2007, 2008, 2009, 2010, 2011, 2012 

// Was adapted for Odyssey TRX, David Fainitski 2015

/* 
Code sends configuration data to the TVL320 via its SPI interface.
Once configured the code loops looking for a change in the boost bit.
If found the new data is sent to the TLV320.

Data to send to TLV320 is 

Common settings
 0x1b    0x1E 0x00 - Reset chip
 0x1b    0x12 0x01 - set digital interface active
 0x1b    0x0C 0x00 - All chip power on
 0x1b    0x0E 0x02 - Slave, 16 bit, I2S
 0x1b    0x10 0x00 - 48k, Normal mode
 0x1b    0x0A 0x00 - turn D/A mute off
 0x1b    0x00 0x00 - set Line in gain to 0

For mic input and boost on/off
 0x1b    0x08 0x14/15 - D/A on, mic input, mic 20dB boost on/off
*/

module sound_codec_init(
	input MCLK,       //12.288 MHz
	input eth_nRESET,
	output reg c_CS,
	output c_MODE,
	output reg c_SDOUT,
	output reg c_SCLK,
	input mic_boost
	);

	
	
	reg   [3:0] data_load;
   reg   [3:0] TLV;
   reg  [15:0] TLV_data;
   reg   [3:0] bit_cnt;
   reg         prev_boost;

// Set up TLV320 data to send 
always @(data_load, mic_boost)	
   begin
     case (data_load)
      4'd0: TLV_data = 16'h1E00;  			// data to load into TLV320
      4'd1: TLV_data = 16'h1201;
      4'd2: TLV_data = 16'h0814 + mic_boost;	// D/A on, MIC on
      4'd3: TLV_data = 16'h0C00;
      4'd4: TLV_data = 16'h0E02; // I2S mode, 02 for I2S, 13 for DSP
      4'd5: TLV_data = 16'h1000;
      4'd6: TLV_data = 16'h0A00;
      4'd7: TLV_data = 16'b0;				// set line in gain
     default: TLV_data = 0;
     endcase
end

	
	// State machine to send data to TLV320 via SPI interface

assign c_MODE = 1'b1;							// Set to 1 for SPI mode

reg [23:0] tlv_timeout;

always @ (posedge MCLK)		
begin
  if (tlv_timeout != (200*12288))        // 200mS @CMCLK= 12.288Mhz
    tlv_timeout <= tlv_timeout + 1'd1;

  case (TLV)
  4'd0:
  begin
    c_CS <= 1'b1;        					// set TLV320 CS high
    bit_cnt <= 4'd15;   					// set starting bit count to 15
    if (tlv_timeout == (200*12288)) 	// wait for 200mS timeout
      TLV <= 4'd1;
    else
      TLV <= 4'd0;
  end

  4'd1:
  begin
    c_CS  <= 1'b0;                		// start data transfer with nCS low
    c_SDOUT <= TLV_data[bit_cnt];  			// set data up
    TLV  <= 4'd2;
  end

  4'd2:
  begin
    c_SCLK <= 1'b1;               			// clock data into TLV320
    TLV  <= 4'd3;
  end

  4'd3:
  begin
    c_SCLK <= 1'b0;               			// reset clock
    TLV  <= 4'd4;
  end

  4'd4:
  begin
    if (bit_cnt == 0) 						// word transfer is complete, check for any more
      TLV <= 4'd5;
    else
    begin
      bit_cnt <= bit_cnt - 1'b1;
      TLV <= 4'd1;    						// go round again
    end
    begin 
    prev_boost <= mic_boost; 	   			// save the current boost setting 
    end 
  end

  4'd5:
  begin
    if (data_load == 7) begin					// stop when all data sent, and wait for boost to change
		c_CS <= 1'b1;        					// set CS high               
		  if (mic_boost != prev_boost ) begin  // has boost or line in or line-in gain changed?
			data_load <= 0;
			TLV <= 4'd0;
		  end
	      else TLV <= 4'd5;     			// hang out here forever
	end
    else begin									// else get next data             	
      TLV  <= 4'd0;           
      data_load <= data_load + 3'b1;  				// select next data word to send
    end
  end
  
  default: TLV <= 4'd0;
  endcase
end

endmodule
