//////////////////////////////////////////////////////////////////////////////////
//  James C. Ahlstrom, 25 June 2009
//
//  Multiple Modifications for Odyssey TRX by David Fainitski 06.07.2015
//  1.4 V N2ADR source based
//  Target device Cyclone 4
//  
//  Reversed back to HiqSDR by David Fainitski, N7DDC 18.05.2020
//  Target device Cyclone 3
//
//////////////////////////////////////////////////////////////////////////////////

module Transceiver(		
	input key_dash,		
	input key_dot,		
	// ADC interface
	input adc_clock,		
	input [13:0] adc_data,
	output adc_reset,
	input adc_overrange,
	// Ethernet controller interface bus
	output [6:0] eth_address_bus,
	inout  [15:0] eth_data_bus,
	output eth_nRD,
	output eth_nWR,
	output eth_nCS,
	output eth_FIFO_SEL,
	output eth_nRESET,
	input  eth_IRQ,
	// Output level control of DAC
	output [7:0] level_data,
	output level_nCS,
	output level_nWR,
	// DAC data
	input clock, // TX DAC clock input	
	output signed [13:0] dac_data,
	// Additional control bits 
	output led_red,
	output  [5:0] Conn_X1,
	input ptt_input,	
	output reg [4:0] attenuate,
	input STR,
	output ANTswitch,
	output CCTX,
	output CCRX,
	output [3:0] Reserve,
	output [7:0] AUX_IO,
	output [7:0] LED
	);
	
	//
   assign eth_nRESET = reset;
	assign adc_reset =  ~eth_nRESET;	// the ADC requires a reset pulse 2 usec
   wire tune = tx_ctrl[6];	// Tune 
	assign Reserve[0] = tune;
	assign Reserve[1] = cl_7;
   assign AUX_IO[0] = PWM_1;
   assign AUX_IO[1] = PWM_2;
	assign CCTX = ~PTT_out;
	assign CCRX = PTT_out;
	wire PTT_led = PTT_out | vna_count != 0;
	assign led_red = clipping || PTT_led; 
	assign AUX_IO[5] = led_red; // led_red_2
	//
   parameter ext_clock_freq = 122880000;		// clock frequency in Hertz
   parameter eth_clock_freq = ext_clock_freq / 2;
   parameter slow_clock_freq = ext_clock_freq / 1280;
   parameter VERSION = 8'd140;	// Be sure to change the version if it matters
	//
	//*************************************************************************
	//                  PLLs
	//*************************************************************************
	// Use a phase locked loop PLL to generate the ADC clock based on adc_clock
	wire rx_clock;			// clock for ADC, etc.
	wire half_clock;		// half speed clock for ethernet
	wire pll_rx_locked;		// PLL lock indicator
	wire slow_clock;		// 96kHz slow clock for key delays, etc.
	pll_rx prx (adc_clock, rx_clock, half_clock, slow_clock, pll_rx_locked);
	
	// Use a phase locked loop PLL to generate the DAC clock based on input clock
	wire tx_clock;			// clock for transmit
	wire pll_tx_locked;		// PLL lock indicator
	pll_tx ptx (clock, tx_clock, pll_tx_locked); 

	//****************************************************************************
	
	// This module resets everything on power start.
	wire reset;
	ethernet_reset ereset (slow_clock, reset);
	//
	wire [7:0] tx_ctrl;  
	// These are the tx_ctrl bits:
	//	0: enable CW transmit
	//	1: enable all other transmit
	//	2: use the HiQSDR extended IO pins not present in the 2010 QEX ver 1.0
	//	3: the key is down (software PTT)
	// 6: TUNE mode
	wire pttX = key_ctrl[8*0 + 3];
	wire [1:0] mode = key_ctrl[8*2 +: 2];
	
	wire PTT_CW = tx_ctrl[0] && tx_en ? ( mode == 0 ? key_dot_db || pttX || dotX || PTT_delay : pttX ) || tx_ctrl[3] || ptt_input_db : 1'd0;
	wire CW_EN =  tx_ctrl[0] && tx_en ?  key_out_d  : 1'd0;
	wire PTT_SSB = tx_ctrl[1] && tx_en ? key_dot_db || key_dash_db || pttX || tx_ctrl[3] || ptt_input_db : 1'd0;
	wire ptt_hpsdr = tx_ctrl[0] ? (mode == 0 ? key_dot_db : 1'd0) || ptt_input_db || PTT_hang || PTT_delay : key_dot_db || key_dash_db || ptt_input_db ;
	wire key_down = key_dot_db || key_dash_db || PTT_CW || PTT_SSB || PTT_delay; // True to transmit any mode
	//
	
	wire demo = key_ctrl[8*2 + 2];
	wire PTT_out = tx_en ? (~demo ? PTT_CW || PTT_SSB || PTT_hang || PTT_delay : 1'd0) : 1'd0;
	//
   wire paddle_swap  = key_ctrl[8*4 + 1];	
	wire dash_input = paddle_swap ? key_dot : key_dash ;
	wire dot_input  = paddle_swap ? key_dash  : key_dot ;
	// Generate transmit and receive keys with delays
	wire  key_dash_db, key_dot_db, ptt_input_db;
	key_debounce kdb (slow_clock, dash_input, dot_input, ~ptt_input, key_dash_db, key_dot_db, ptt_input_db);
	
	//--------------------------------------------------------------------------------------------
   //  	Iambic CW Keyer
   //--------------------------------------------------------------------------------------------
   // parameter is clock speed in kHz. Keyer data from PC is sent in I channel when in FPGA CW mode. 
   wire key_out;
	wire dotX = key_ctrl[8*0 + 2];
	wire dashX = key_ctrl[8*0 + 1];
	wire dash = key_dash_db || dashX || h_dash;
   wire dot  = key_dot_db || dotX || h_dot;
   wire PTT_delay;	
   iambic ia (slow_clock, reset, key_ctrl, key_ctrl_valid, dot, dash, h_CWX, key_out);
	//
	wire key_out_d;
	key_delay kdl (slow_clock, reset, key_ctrl[8*7 +: 8], key_ctrl_valid, key_out, key_out_d, PTT_delay);
	
	// These extended IO bits are used by the HiQSDR:
	wire [31:0] Extend_IO;
	//assign Conn_X1[6]   =  ~ Extend_IO[5] ;// 1- VNA duplex
	assign Conn_X1[5:0] =  Extend_IO[5:0];  
	assign ANTswitch   =  Extend_IO[16]; //AntSwitch

   always @(*) 
	if (vna_count != 0) attenuate <= 5'b10000;// 20db VNA mode
	else
	if (PTT_out)  attenuate <= 5'b11000;  // 30dB PTT mode
	else
	begin                                  // RX mode
      if (hpsdr_mode)            // HPSDR mode
      begin
			if (att_20 == 1) attenuate <= 5'b10000; // Mercury mode -20dB
			else if(att_step >= 14) attenuate <= 5'b11000; // 30 Hermes step attenuator mode
			else if(att_step >= 11) attenuate <= 5'b10010; //  25 Hermes step attenuator mode
			else if(att_step >= 9) attenuate <= 5'b10000; // 20 Hermes step attenuator mode
			else if(att_step >= 6) attenuate <= 5'b01010; // 15 Hermes step attenuator mode
			else if(att_step >= 4) attenuate <= 5'b01000; // 10 Hermes step attenuator mode
			else if(att_step >= 2) attenuate <= 5'b00010; // 5 Hermes step attenuator mode
			else attenuate <= 5'b00000; // 0
      end
      else   // HiqSDR mode
            attenuate = Extend_IO[12:8]; 	
	end
   //
	
	level_calibration tx_cal (slow_clock, reset, level_nCS, level_nWR);
	
	assign LED[7] = ~ (pll_rx_locked && pll_tx_locked);		// LED[7]: PLL not locked
	wire clipping;
	clip_led cl (slow_clock, adc_overrange, clipping);	// turn on LED for ADC clip

	// Module to handle Ethernet bus transactions
	wire [31:0] rx_tune_phase;
	wire [31:0] rx_tune_phase_2;
	wire [7:0] rx_ctrl, rx_ctrl_2;
	wire [31:0] tx_tune_phase;
	wire tx_en;
	wire [111:0] key_ctrl;
	wire key_ctrl_valid;
	wire hpsdr_mode;
	wire att_20, att_step_en, cl_7;
	wire [3:0] att_step;
	wire two_receiver, h_dot, h_dash, h_CWX;
	wire get_next_sample = PTT_SSB ? get_next_dac : get_next_cdc;
	wire [15:0] vna_count;		// number of VNA scan data points; zero for non-VNA operation
	wire srv_data_wr;
	ethernet mod_eth (half_clock, slow_clock, key_down, clipping,
		rx_ctrl, rx_ctrl_2, rx_tune_phase, rx_tune_phase_2, tx_ctrl, tx_tune_phase,
		rx_mem_read_addr, rx_mem_q, rx_mem_block, rx_mem_read_addr_2, rx_mem_q_2, rx_mem_block_2,
		tx_sample, get_next_sample,
		eth_address_bus, eth_data_bus, eth_nRD, eth_nWR,
		eth_nCS, eth_FIFO_SEL, eth_nRESET, eth_IRQ,
	   level_data, Extend_IO, vna_count, tx_en, st_volume, 
		key_ctrl, key_ctrl_valid,
		hpsdr_mode, rawRamReset, rawRamReady, rawRamReadAddr, rawRamOut,
		att_20, cl_7, att_step, att_step_en, two_receiver,	h_dot, h_dash, h_CWX
		);
			
	// Memory for ADC data
	wire rx_mem_block;				// Index of available block 0 or 1
	wire [31:0] rx_mem_data;		// data to write
	wire [31:0] rx_mem_q;			// memory data at read address
	wire [9:0]  rx_mem_read_addr;	// read address
	wire [9:0]  rx_mem_write_addr;	// write address
	wire rx_mem_wen;				// write enable
	eth_ram1 adc_ram (rx_mem_data, rx_mem_read_addr, half_clock,
		               rx_mem_write_addr, rx_clock, rx_mem_wen, rx_mem_q);	
		
	// Memory for 2-nd receiver ADC data
	wire rx_mem_block_2;				// Index of available block 0 or 1
	wire [31:0] rx_mem_data_2;		// data to write
	wire [31:0] rx_mem_q_2;			// memory data at read address
	wire [9:0]  rx_mem_read_addr_2;	// read address
	wire [9:0]  rx_mem_write_addr_2;	// write address
	wire rx_mem_wen_2;				// write enable
	eth_ram1 adc_ram_2 (rx_mem_data_2, rx_mem_read_addr_2, half_clock,
		               rx_mem_write_addr_2, rx_clock, rx_mem_wen_2, rx_mem_q_2);
							
	// read and register ADC data
	reg [15:0] reg_adc_data;
	always @(posedge rx_clock)
	begin
		reg_adc_data <= {adc_data[13], adc_data[12:0], 2'b00};
	end
   	
	// The Receiver
	wire [31:0] vna_tune_phase;
	Receiver rx (rx_clock, reg_adc_data, reset, rx_tune_phase, rx_tune_phase_2, tx_tune_phase, vna_tune_phase, rx_ctrl, rx_ctrl_2,
		rx_mem_data, rx_mem_write_addr, rx_mem_wen, rx_mem_block, rx_mem_data_2, rx_mem_write_addr_2, rx_mem_wen_2, 
		rx_mem_block_2, vna_count, tx_cordic_out_i, tx_cordic_out_q, hpsdr_mode, ptt_hpsdr, key_dash_db, key_dot_db, clipping, two_receiver);
		
	// DAC output can be either the transmitter, or for VNA, the receiver reference RF
	assign dac_data = PTT_out || vna_count != 0 ? tx_dac_data : 14'd0;
	
	// The Transmitter
	wire signed [31:0] tx_sample;		// next sample to transmit
	wire get_next_dac;					// toggle to get next sample
	wire signed [13:0] tx_dac_data;		// DAC data from the transmitter
	wire cw_on, PTT_hang;
	wire signed [15:0] tx_cordic_out_i, tx_cordic_out_q;
	Transmitter tx (tx_clock, CW_EN, PTT_SSB, reset, tx_tune_phase, vna_tune_phase, vna_count, 
	                tx_cordic_out_i, tx_cordic_out_q, tx_ctrl[7:0], 
		              tx_sample, get_next_dac, tx_dac_data, STR, slow_clock, cw_on, PTT_hang, key_ctrl, key_ctrl_valid);
		
		
	// The sound codec module
	wire get_next_cdc;	// toggle to get next sample
	wire [7:0] st_volume;
	wire st_on = key_out && tx_ctrl[0];
	wire PWM_1, PWM_2;
	sound_codec codec (rx_clock, reset, tx_sample, get_next_cdc, 
	                    slow_clock, st_on, st_volume, PWM_1, PWM_2,
							  key_ctrl, key_ctrl_valid, PTT_SSB, PTT_CW, tune, tx_en);
	//
	// Wideband spectrum data
	wire [11:0] rawRamReadAddr;
	wire [15:0] rawRamOut;
	wire rawRamReset, rawRamReady;
	band_scope bs ( rx_clock, half_clock, reg_adc_data, rawRamReadAddr, rawRamOut, rawRamReset, rawRamReady);
	
endmodule







