//////////////////////////////////////////////////////////////////////////////////
//             Odyssey TRX - David Fainitski  04/07/2015
//
//////////////////////////////////////////////////////////////////////////////////

module Transmitter(
	input clock,
	input CW_EN,
	input PTT_SSB,
	input reset,
	input [31:0] tx_tune_phase,
	input [31:0] vna_tune_phase,
	input [15:0] vna_count,
	output signed [15:0] cordic_out_i,
	output signed[15:0] cordic_out_q,
	input [7:0] tx_ctrl,
	input signed [31:0] tx_sample,		// next sample to transmit
	output reg get_next_dac,			// toggle to get next sample
	output signed [13:0] dac_data,
	input STR,
	input pro_clock,                  //96kHz for CW profile
	output reg cw_on,
	output reg PTT_hang,
	input [111:0] key_ctrl,
	input key_ctrl_valid
	);

	
	// Interpolate I/Q samples in memory from 48 kHz to the clock frequency
	wire req1, req2;
	wire signed [19:0] y1_r, y1_i, y2_r, y2_i;
	FirInterp8 fi (clock, req2, req1, tx_sample[31:16], tx_sample[15:0], y1_r, y1_i);
	CicInterpM5 #(.RRRR(320), .IBITS(20), .OBITS(20), .GBITS(34))
        	in2 (clock, 1'd1, req2, y1_r, y1_i, y2_r, y2_i);

	// Tune transmitter with CORDIC
	wire signed [15:0] tx_real, tx_imag;	// input to cordic	
	wire [31:0] tune_phase = vna_count != 0 ? vna_tune_phase : tx_tune_phase;
	cpl_cordic #(.OUT_WIDTH(16))
 		cordic_inst (.clock(clock), .frequency(tune_phase), .in_data_I(tx_imag),			
		.in_data_Q(tx_real), .out_data_I(cordic_out_i), .out_data_Q(cordic_out_q));	
	
	assign dac_data = STR ? {~cordic_out_i[15], cordic_out_i[14:2]} : cordic_out_i[15:2];
	
	assign tx_real =  vna_count != 0  ? 16'd8192 : ( (cw_on_f & tx_ctrl[0]) ? cw_level : (ssb_en ? y2_r[19:4] : 1'd0) );  
	assign tx_imag =  vna_count != 0  ? 16'd0    : ( (cw_on_f & tx_ctrl[0]) ? 16'd0    : (ssb_en ? y2_i[19:4] : 1'd0) );
	
	// CW level normalise. 
   wire [15:0] cw_level;	
	parameter MAX_CW_LEVEL = 16'd39000;	// 	
	mult_16x16_uns cw_mult (MAX_CW_LEVEL, cw_profile, cw_level);
    
	reg [3:0] tx_state =  0;
	parameter txOff     = 0;
	parameter txCheck   = 1;
	parameter txPTT_CW  = 2;
	parameter txCW_EN   = 3;
	parameter txPtt     = 4;
	//
	always @(posedge clock)
	begin
		    case (tx_state)
			 	txOff :   begin
				             cw_on <= 0;
	                      if (!cw_on_f)  tx_state <= txCheck; 
                      end	
			   txCheck : begin  // Checking for TX mode
								 if (CW_EN ) tx_state <= txCW_EN;  // Transmitt CW
								 if (PTT_SSB) tx_state <= txPtt;
				          end
				txCW_EN : begin // Transmitt CW
                          cw_on <= 1;   // KEY
				              if (!CW_EN) tx_state <= txOff; 
                    end 
				txPtt :begin // SSB PTT
                       if (req1) get_next_dac <= ~ get_next_dac;   //SSB
							  if (!PTT_SSB )  tx_state <= txOff;
                   end	 				
				default: tx_state <= txOff;
			 endcase
	end
	

	wire [15:0] cw_profile;
	reg [9:0] pro_count;
	wire cw_on_f = (pro_count != 0);
	profile cw_pro (.address(pro_count), .clock(pro_clock), .q(cw_profile));
	
   reg [7:0] delay = 25;
	reg [9:0] hang  = 230; 
	always @(posedge key_ctrl_valid)
	begin
	   if (reset)
		   begin
	        delay <= key_ctrl[8*7 +: 8];
		     hang  <= key_ctrl[8*8 +: 8] * 4'd10; 
			end
	end

	// SSB mode delay
	reg ssb_en = 0;
	reg  [16:0] ssb_timer = 0;	
	always @(posedge pro_clock)
	begin
	   if (PTT_SSB == 0) begin ssb_en <= 0; ssb_timer <= (delay + 1'd1) * 7'd96; end
		else
		begin
	      if (ssb_timer != 0) ssb_timer <= ssb_timer - 1'd1; 
	      else ssb_en <= 1'd1;
		end	
	end
	
	

reg  [3:0]prof_state = 0;
reg [16:0] hang_timer = 0;  // Max delay = 1023mS so need 96 * 1023 ~= 200,000 = 2^17 counts. 

always @ (posedge pro_clock)
begin 
if(!reset) begin PTT_hang <= 0; hang_timer <= 1'd0; prof_state <= 0; pro_count <= 1'd0; end
else
case (prof_state)
0: 	begin
			hang_timer <= 0;			
		   if (cw_on) begin PTT_hang <= 1; prof_state <= 1; end		// so go straight to profie generation			
		end
1:	begin
		if (pro_count != 511) pro_count <= pro_count + 1'b1;
		else
		if (!cw_on) prof_state <= 2; 			// stay in this state until key is released	
   end		
2: begin	
		if (pro_count != 0) pro_count <= pro_count - 1'b1;	// ROM address counter runs backwards	
		else prof_state <= 3;
	end
3: begin
		if (cw_on) prof_state <= 0; 
		else 
		begin
		   if (hang_timer != (hang * 96)) hang_timer <= hang_timer + 1'd1;
		   else begin PTT_hang <= 0; prof_state <= 0; end	
		end 
   end
endcase
end



endmodule
